""" boris.models

    The boris.models module provides a collection of 
    models (such as ResNet) which can easily be used 
    for self-supervised learning.

"""

from ._resnet_simclr import ResNetSimCLR
from ._resnet_simclr import ResNetGenerator
